@@ -1,7 +1,7 @@
 #ifndef _averages_hpp_INCLUDED
 #define _averages_hpp_INCLUDED
 
-#include "ema.hpp"              // alphabetically after 'averages.hpp'
+#include "ema.hpp" // alphabetically after 'averages.hpp'
 
 namespace CaDiCaL {
 
@@ -21,16 +21,15 @@ struct Averages {
       EMA slow; // average slow (large window) moving trail level
     } trail;
 
-    EMA size;   // average learned clause size
-    EMA jump;   // average (potential non-chronological) back-jump level
-    EMA level;  // average back track level after conflict
+    EMA size;  // average learned clause size
+    EMA jump;  // average (potential non-chronological) back-jump level
+    EMA level; // average back track level after conflict
 
   } current, saved;
 
-  Averages () : swapped (0) { }
-
+  Averages () : swapped (0) {}
 };
 
-}
+} // namespace CaDiCaL
 
 #endif
