@@ -355,6 +355,8 @@ int App::main (int argc, char **argv) {
       return 0;
     } else if (!strcmp (arg, "--copyright")) {
       printf ("%s\n", copyright ());
+      printf ("%s\n", authors ());
+      printf ("%s\n", affiliations ());
       return 0;
     }
   }
@@ -613,6 +615,10 @@ int App::main (int argc, char **argv) {
                      tout.normal_code ());
     solver->message ("%s%s%s", tout.bright_magenta_code (), copyright (),
                      tout.normal_code ());
+    solver->message ("%s%s%s", tout.bright_magenta_code (), authors (),
+                     tout.normal_code ());
+    solver->message ("%s%s%s", tout.bright_magenta_code (), affiliations (),
+                     tout.normal_code ());
     solver->message ();
     CaDiCaL::Solver::build (stdout, "c ");
   }
