@@ -94,6 +94,7 @@ Clause *Internal::new_clause (bool red, int glue) {
 
   size_t bytes = Clause::bytes (size);
   Clause *c = (Clause *) new char[bytes];
+  DeferDeleteArray<char> clause_delete ((char *) c);
 
   c->id = ++clause_id;
 
@@ -140,6 +141,7 @@ Clause *Internal::new_clause (bool red, int glue) {
   }
 
   clauses.push_back (c);
+  clause_delete.release ();
   LOG (c, "new pointer %p", (void *) c);
 
   if (likely_to_be_kept_clause (c))
@@ -189,7 +191,8 @@ void Internal::promote_clause (Clause *c, int new_glue) {
 // (aligned) removed bytes, resulting from shrinking the clause.
 //
 size_t Internal::shrink_clause (Clause *c, int new_size) {
-
+  if (opts.check && is_external_forgettable (c->id))
+    mark_garbage_external_forgettable (c->id);
   assert (new_size >= 2);
   int old_size = c->size;
   assert (new_size < old_size);
@@ -286,6 +289,12 @@ void Internal::mark_garbage (Clause *c) {
     proof->delete_clause (c);
   }
 
+  // Because of the internal model checking, external forgettable clauses
+  // must be marked as removed already upon mark_garbage, can not wait until
+  // actual deletion.
+  if (opts.check && is_external_forgettable (c->id))
+    mark_garbage_external_forgettable (c->id);
+
   assert (stats.current.total > 0);
   stats.current.total--;
 
@@ -329,7 +338,8 @@ void Internal::assign_original_unit (uint64_t id, int lit) {
   trail.push_back (lit);
   num_assigned++;
   const unsigned uidx = vlit (lit);
-  unit_clauses[uidx] = id;
+  if (lrat || frat)
+    unit_clauses (uidx) = id;
   LOG ("original unit assign %d", lit);
   assert (num_assigned == trail.size () || level);
   mark_fixed (lit);
@@ -349,6 +359,8 @@ void Internal::assign_original_unit (uint64_t id, int lit) {
 // sometimes the pointer to the new clause is needed, therefore it is
 // made sure that newest_clause points to the new clause upon return.
 //
+// TODO: Find another name for 'tainted' in the context of ilb, tainted
+// is reconstruction related already and they should not mix.
 void Internal::add_new_original_clause (uint64_t id) {
 
   if (!from_propagator && level && !opts.ilb) {
@@ -367,7 +379,7 @@ void Internal::add_new_original_clause (uint64_t id) {
   size_t unassigned = 0;
   newest_clause = 0;
   if (unsat) {
-    LOG ("skipping clause since formula already inconsistent");
+    LOG ("skipping clause since formula is already inconsistent");
     skip = true;
   } else {
     assert (clause.empty ());
@@ -387,7 +399,7 @@ void Internal::add_new_original_clause (uint64_t id) {
             int elit = externalize (lit);
             unsigned eidx = (elit > 0) + 2u * (unsigned) abs (elit);
             if (!external->ext_units[eidx]) {
-              uint64_t uid = (unit_clauses[vlit (-lit)]);
+              uint64_t uid = (unit_clauses (vlit (-lit)));
               assert (uid);
               lrat_chain.push_back (uid);
             }
@@ -410,8 +422,15 @@ void Internal::add_new_original_clause (uint64_t id) {
       unmark (lit);
   }
   if (skip) {
-    if (from_propagator)
+    if (from_propagator) {
       stats.ext_prop.elearn_conf++;
+
+      // In case it was a skipped external forgettable, we need to mark it
+      // immediately as removed
+
+      if (opts.check && is_external_forgettable (id))
+        mark_garbage_external_forgettable (id);
+    }
     if (proof) {
       proof->delete_external_original_clause (id, false, external->eclause);
     }
@@ -428,6 +447,14 @@ void Internal::add_new_original_clause (uint64_t id) {
                                                 external->eclause);
       }
       external->check_learned_clause ();
+
+      if (from_propagator) {
+        // The original form of the added clause is immediately forgotten
+        // TODO: shall we save and check the simplified form? (one with
+        // new_id)
+        if (opts.check && is_external_forgettable (id))
+          mark_garbage_external_forgettable (id);
+      }
     }
     external->eclause.clear ();
     lrat_chain.clear ();
@@ -454,7 +481,8 @@ void Internal::add_new_original_clause (uint64_t id) {
         v.level = 0;
         v.reason = 0;
         const unsigned uidx = vlit (clause[0]);
-        unit_clauses[uidx] = new_id;
+        if (lrat || frat)
+          unit_clauses (uidx) = new_id;
         mark_fixed (clause[0]);
       } else {
         const int lit = clause[0];
@@ -466,14 +494,14 @@ void Internal::add_new_original_clause (uint64_t id) {
         assign_original_unit (new_id, lit);
       }
     } else {
-      move_literal_to_watch (false);
-      move_literal_to_watch (true);
+      move_literals_to_watch ();
 #ifndef NDEBUG
       check_watched_literal_invariants ();
 #endif
       int glue = (int) (learned_levels.size () + unassigned);
       assert (glue <= (int) clause.size ());
-      Clause *c = new_clause (false, glue);
+      bool clause_redundancy = from_propagator && ext_clause_forgettable;
+      Clause *c = new_clause (clause_redundancy, glue);
       c->id = new_id;
       clause_id--;
       watch_clause (c);
@@ -555,10 +583,10 @@ Clause *Internal::new_clause_as (const Clause *orig) {
 //
 Clause *Internal::new_resolved_irredundant_clause () {
   external->check_learned_clause ();
-  Clause *res = new_clause (false);
   if (proof) {
-    proof->add_derived_clause (res, lrat_chain);
+    proof->add_derived_clause (clause_id + 1, false, clause, lrat_chain);
   }
+  Clause *res = new_clause (false);
   assert (!watching ());
   return res;
 }
