@@ -300,7 +300,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
       satisfied = lit;
       break;
     } else if (tmp < 0) {
-      if (!opts.lrat || opts.lratexternal)
+      if (!lrat)
         continue;
       Flags &f = flags (lit);
       if (f.seen)
@@ -308,7 +308,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
       analyzed.push_back (lit);
       f.seen = true;
       const unsigned uidx = vlit (-lit);
-      uint64_t id = unit_clauses[uidx];
+      uint64_t id = unit_clauses (uidx);
       assert (id);
       lrat_chain.push_back (id);
       continue;
@@ -341,7 +341,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
       satisfied = lit;
       break;
     } else if (tmp < 0) {
-      if (!opts.lrat || opts.lratexternal)
+      if (!lrat)
         continue;
       Flags &f = flags (lit);
       if (f.seen)
@@ -349,7 +349,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
       analyzed.push_back (lit);
       f.seen = true;
       const unsigned uidx = vlit (-lit);
-      uint64_t id = unit_clauses[uidx];
+      uint64_t id = unit_clauses (uidx);
       assert (id);
       lrat_chain.push_back (id);
       continue;
@@ -366,7 +366,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
   unmark (c);
   const int64_t size = clause.size ();
 
-  if (opts.lrat && !opts.lratexternal) {
+  if (lrat) {
     lrat_chain.push_back (d->id);
     lrat_chain.push_back (c->id);
   }
@@ -412,7 +412,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
   }
 
   LOG (clause, "resolvent");
-  assert (!opts.lrat || opts.lratexternal || !lrat_chain.empty ());
+  assert (!lrat || !lrat_chain.empty ());
 
   // Double self-subsuming resolution.  The clauses 'c' and 'd' are
   // identical except for the pivot which occurs in different phase.  The
@@ -422,7 +422,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
     assert (s == size + 1);
     assert (t == size + 1);
     clause.clear ();
-    // TODO: lrat is c + d (+ eventual units)
+    // TODO: LRAT is c + d (+ eventual units)
     elim_on_the_fly_self_subsumption (eliminator, c, pivot);
     LOG (d, "double pivot %d on-the-fly self-subsuming resolution", -pivot);
     stats.elimotfsub++;
@@ -439,7 +439,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
   if (s > size) {
     assert (s == size + 1);
     clause.clear ();
-    // TODO: lrat is c + d (+ eventual units) -> not 100% sure.
+    // TODO: LRAT is c + d (+ eventual units) -> not 100% sure.
     elim_on_the_fly_self_subsumption (eliminator, c, pivot);
     return false;
   }
@@ -449,7 +449,7 @@ bool Internal::resolve_clauses (Eliminator &eliminator, Clause *c,
   if (t > size) {
     assert (t == size + 1);
     clause.clear ();
-    // TODO: lrat is c + d (+ eventual units) -> same.
+    // TODO: LRAT is c + d (+ eventual units) -> same.
     elim_on_the_fly_self_subsumption (eliminator, d, -pivot);
     return false;
   }
@@ -578,7 +578,7 @@ inline void Internal::elim_add_resolvents (Eliminator &eliminator,
         continue;
       if (!resolve_clauses (eliminator, c, pivot, d, false))
         continue;
-      assert (!opts.lrat || opts.lratexternal || !lrat_chain.empty ());
+      assert (!lrat || !lrat_chain.empty ());
       Clause *r = new_resolved_irredundant_clause ();
       elim_update_added_clause (eliminator, r);
       eliminator.enqueue (r);
@@ -598,8 +598,8 @@ inline void Internal::elim_add_resolvents (Eliminator &eliminator,
 // Remove clauses with 'pivot' and '-pivot' by marking them as garbage and
 // push them on the extension stack.
 
-void Internal::mark_eliminated_clauses_as_garbage (Eliminator &eliminator,
-                                                   int pivot) {
+void Internal::mark_eliminated_clauses_as_garbage (
+    Eliminator &eliminator, int pivot, bool &deleted_binary_clause) {
   assert (!unsat);
 
   LOG ("marking irredundant clauses with %d as garbage", pivot);
@@ -614,14 +614,18 @@ void Internal::mark_eliminated_clauses_as_garbage (Eliminator &eliminator,
   for (const auto &c : ps) {
     if (c->garbage)
       continue;
-    mark_garbage (c);
     assert (!c->redundant);
     if (!substitute || c->gate) {
+      if (proof)
+        proof->weaken_minus (c);
+      if (c->size == 2)
+        deleted_binary_clause = true;
       external->push_clause_on_extension_stack (c, pivot);
 #ifndef NDEBUG
       pushed++;
 #endif
     }
+    mark_garbage (c);
     elim_update_removed_clause (eliminator, c, pivot);
   }
   erase_occs (ps);
@@ -632,14 +636,18 @@ void Internal::mark_eliminated_clauses_as_garbage (Eliminator &eliminator,
   for (const auto &d : ns) {
     if (d->garbage)
       continue;
-    mark_garbage (d);
     assert (!d->redundant);
     if (!substitute || d->gate) {
+      if (proof)
+        proof->weaken_minus (d);
+      if (d->size == 2)
+        deleted_binary_clause = true;
       external->push_clause_on_extension_stack (d, -pivot);
 #ifndef NDEBUG
       pushed++;
 #endif
     }
+    mark_garbage (d);
     elim_update_removed_clause (eliminator, d, -pivot);
   }
   erase_occs (ns);
@@ -656,8 +664,8 @@ void Internal::mark_eliminated_clauses_as_garbage (Eliminator &eliminator,
 
 // Try to eliminate 'pivot' by bounded variable elimination.
 
-void Internal::try_to_eliminate_variable (Eliminator &eliminator,
-                                          int pivot) {
+void Internal::try_to_eliminate_variable (Eliminator &eliminator, int pivot,
+                                          bool &deleted_binary_clause) {
 
   if (!active (pivot))
     return;
@@ -701,7 +709,8 @@ void Internal::try_to_eliminate_variable (Eliminator &eliminator,
       LOG ("number of resolvents on %d are bounded", pivot);
       elim_add_resolvents (eliminator, pivot);
       if (!unsat)
-        mark_eliminated_clauses_as_garbage (eliminator, pivot);
+        mark_eliminated_clauses_as_garbage (eliminator, pivot,
+                                            deleted_binary_clause);
       if (active (pivot))
         mark_eliminated (pivot);
     } else
@@ -744,7 +753,7 @@ void Internal::
 // variables have been tried).  Otherwise it was asynchronously terminated
 // or the resolution limit was hit.
 
-int Internal::elim_round (bool &completed) {
+int Internal::elim_round (bool &completed, bool &deleted_binary_clause) {
 
   assert (opts.elim);
   assert (!unsat);
@@ -869,7 +878,7 @@ int Internal::elim_round (bool &completed) {
     int idx = schedule.front ();
     schedule.pop_front ();
     flags (idx).elim = false;
-    try_to_eliminate_variable (eliminator, idx);
+    try_to_eliminate_variable (eliminator, idx, deleted_binary_clause);
 #ifndef QUIET
     tried++;
 #endif
@@ -990,6 +999,11 @@ void Internal::elim (bool update_limits) {
   PHASE ("elim-phase", stats.elimphases,
          "starting at most %d elimination rounds", opts.elimrounds);
 
+  if (external_prop) {
+    assert (!level);
+    private_steps = true;
+  }
+
 #ifndef QUIET
   int old_active_variables = active ();
   int old_eliminated = stats.all.eliminated;
@@ -1014,7 +1028,7 @@ void Internal::elim (bool update_limits) {
   // variable elimination phase ('elim') ran until completion.  This
   // potentially triggers an incremental increase of the elimination bound.
   //
-  bool phase_complete = false;
+  bool phase_complete = false, deleted_binary_clause = false;
 
   int round = 1;
 
@@ -1025,7 +1039,7 @@ void Internal::elim (bool update_limits) {
 #ifndef QUIET
     int eliminated =
 #endif
-        elim_round (round_complete);
+        elim_round (round_complete, deleted_binary_clause);
 
     if (!round_complete) {
       PHASE ("elim-phase", stats.elimphases, "last round %d incomplete %s",
@@ -1076,6 +1090,8 @@ void Internal::elim (bool update_limits) {
            stats.elimcompleted + 1, lim.elimbound);
   }
 
+  if (deleted_binary_clause)
+    delete_garbage_clauses ();
   init_watches ();
   connect_watches ();
 
@@ -1102,6 +1118,11 @@ void Internal::elim (bool update_limits) {
          eliminated, percent (eliminated, old_active_variables));
 #endif
 
+  if (external_prop) {
+    assert (!level);
+    private_steps = false;
+  }
+
   if (!update_limits)
     return;
 
