@@ -7,6 +7,14 @@ void External::push_zero_on_extension_stack () {
   LOG ("pushing 0 on extension stack");
 }
 
+void External::push_id_on_extension_stack (uint64_t id) {
+  const uint32_t higher_bits = static_cast<int> (id << 32);
+  const uint32_t lower_bits = (id & (((uint64_t) 1 << 32) - 1));
+  extension.push_back (higher_bits);
+  extension.push_back (lower_bits);
+  LOG ("pushing id %" PRIu64 " = %d + %d", id, higher_bits, lower_bits);
+}
+
 void External::push_clause_literal_on_extension_stack (int ilit) {
   assert (ilit);
   const int elit = internal->externalize (ilit);
@@ -40,6 +48,8 @@ void External::push_clause_on_extension_stack (Clause *c) {
   internal->stats.weakened++;
   internal->stats.weakenedlen += c->size;
   push_zero_on_extension_stack ();
+  push_id_on_extension_stack (c->id);
+  push_zero_on_extension_stack ();
   for (const auto &lit : *c)
     push_clause_literal_on_extension_stack (lit);
 }
@@ -50,13 +60,16 @@ void External::push_clause_on_extension_stack (Clause *c, int pivot) {
   push_clause_on_extension_stack (c);
 }
 
-void External::push_binary_clause_on_extension_stack (int pivot,
+void External::push_binary_clause_on_extension_stack (uint64_t id,
+                                                      int pivot,
                                                       int other) {
   internal->stats.weakened++;
   internal->stats.weakenedlen += 2;
   push_zero_on_extension_stack ();
   push_witness_literal_on_extension_stack (pivot);
   push_zero_on_extension_stack ();
+  push_id_on_extension_stack (id);
+  push_zero_on_extension_stack ();
   push_clause_literal_on_extension_stack (pivot);
   push_clause_literal_on_extension_stack (other);
 }
@@ -64,7 +77,8 @@ void External::push_binary_clause_on_extension_stack (int pivot,
 /*------------------------------------------------------------------------*/
 
 void External::push_external_clause_and_witness_on_extension_stack (
-    const vector<int> &c, const vector<int> &w) {
+    const vector<int> &c, const vector<int> &w, uint64_t id) {
+  assert (id);
   extension.push_back (0);
   for (const auto &elit : w) {
     assert (elit != INT_MIN);
@@ -73,6 +87,11 @@ void External::push_external_clause_and_witness_on_extension_stack (
     mark (witness, elit);
   }
   extension.push_back (0);
+  const uint32_t higher_bits = static_cast<int> (id << 32);
+  const uint32_t lower_bits = (id & (((uint64_t) 1 << 32) - 1));
+  extension.push_back (higher_bits);
+  extension.push_back (lower_bits);
+  extension.push_back (0);
   for (const auto &elit : c) {
     assert (elit != INT_MIN);
     init (abs (elit));
@@ -133,18 +152,27 @@ void External::extend () {
     while ((lit = *--i)) {
       if (satisfied)
         continue;
-      if (ival (lit) > 0)
+      if (ival (lit) == lit)
         satisfied = true;
       assert (i != begin);
     }
     assert (i != begin);
+    LOG ("id=%" PRIu64, ((uint64_t) *i << 32) + *(i - 1));
+    assert (*i || *(i - 1));
+    --i;
+    assert (i != begin);
+    --i;
+    assert (i != begin);
+    assert (!*i);
+    --i;
+    assert (i != begin);
     if (satisfied)
       while (*--i)
         assert (i != begin);
     else {
       while ((lit = *--i)) {
         const int tmp = ival (lit); // not 'signed char'!!!
-        if (tmp < 0) {
+        if (tmp != lit) {
           LOG ("flipping blocking literal %d", lit);
           assert (lit);
           assert (lit != INT_MIN);
@@ -180,11 +208,20 @@ bool External::traverse_witnesses_backward (WitnessIterator &it) {
     int lit;
     while ((lit = *--i))
       clause.push_back (lit);
+    assert (!lit);
+    --i;
+    const uint64_t id =
+        ((uint64_t) *(i - 1) << 32) + static_cast<uint64_t> (*i);
+    assert (id);
+    i -= 2;
+    assert (!*i);
+    assert (i != begin);
     while ((lit = *--i))
       witness.push_back (lit);
     reverse (clause.begin (), clause.end ());
     reverse (witness.begin (), witness.end ());
-    if (!it.witness (clause, witness))
+    LOG (clause, "traversing clause");
+    if (!it.witness (clause, witness, id))
       return false;
     clause.clear ();
     witness.clear ();
@@ -206,9 +243,16 @@ bool External::traverse_witnesses_forward (WitnessIterator &it) {
         witness.push_back (lit);
       assert (!lit);
       assert (i != end);
+      assert (!*i);
+      const uint64_t id =
+          ((uint64_t) *i << 32) + static_cast<uint64_t> (*(i + 1));
+      assert (id > 0);
+      i += 3;
+      assert (*i);
+      assert (i != end);
       while (i != end && (lit = *i++))
         clause.push_back (lit);
-      if (!it.witness (clause, witness))
+      if (!it.witness (clause, witness, id))
         return false;
       clause.clear ();
       witness.clear ();
@@ -219,4 +263,18 @@ bool External::traverse_witnesses_forward (WitnessIterator &it) {
 
 /*------------------------------------------------------------------------*/
 
+void External::conclude_sat () {
+  if (!internal->proof || concluded)
+    return;
+  concluded = true;
+  if (!extended)
+    extend ();
+  vector<int> model;
+  for (int idx = 1; idx <= max_var; idx++) {
+    const int lit = ival (idx);
+    model.push_back (lit);
+  }
+  internal->proof->conclude_sat (model);
+}
+
 } // namespace CaDiCaL
