@@ -4,6 +4,8 @@
 /*------------------------------------------------------------------------*/
 
 #include "range.hpp"
+#include <unordered_map>
+#include <vector>
 
 /*------------------------------------------------------------------------*/
 
@@ -64,7 +66,7 @@ struct External {
   vector<int> constraint;  // External constraint. Terminated by zero.
 
   vector<uint64_t>
-      ext_units; // External units. Needed to compute lrat for eclause
+      ext_units; // External units. Needed to compute LRAT for eclause
   vector<bool> ext_flags; // to avoid duplicate units
   vector<int> eclause;    // External version of original input clause.
   // The extension stack for reconstructing complete satisfying assignments
@@ -74,7 +76,8 @@ struct External {
   // external literals on the 'extension' stack after mapping the
   // internal literals given as arguments with 'externalize'.
 
-  bool extended;         // Have been extended.
+  bool extended; // Have been extended.
+  bool concluded;
   vector<int> extension; // Solution reconstruction extension stack.
 
   vector<bool> witness; // Literal witness on extension stack.
@@ -95,12 +98,22 @@ struct External {
   void export_learned_unit_clause (int ilit);
   void export_learned_large_clause (const vector<int> &);
 
+  // If there is a listener for fixed assignments.
+
+  FixedAssignmentListener *fixed_listener;
+
   // If there is an external propagator.
 
   ExternalPropagator *propagator;
 
   vector<bool> is_observed; // Quick flag for each external variable
 
+  // Saved 'forgettable' original clauses coming from the external
+  // propagator. The value of the map starts with a Boolean flag indicating
+  // if the clause is still present or got already deleted, and then
+  // followed by the literals of the clause.
+  unordered_map<uint64_t, vector<int>> forgettable_original;
+
   void add_observed_var (int elit);
   void remove_observed_var (int elit);
   void reset_observed_vars ();
@@ -109,6 +122,8 @@ struct External {
   bool is_witness (int elit);
   bool is_decision (int elit);
 
+  void force_backtrack (size_t new_level);
+
   //----------------------------------------------------------------------//
 
   signed char *solution; // Given solution checking for debugging.
@@ -163,12 +178,14 @@ struct External {
 
   void push_clause_on_extension_stack (Clause *);
   void push_clause_on_extension_stack (Clause *, int witness);
-  void push_binary_clause_on_extension_stack (int witness, int other);
+  void push_binary_clause_on_extension_stack (uint64_t id, int witness,
+                                              int other);
 
   // The main 'extend' function which extends an internal assignment to an
   // external assignment using the extension stack (and sets 'extended').
   //
   void extend ();
+  void conclude_sat ();
 
   /*----------------------------------------------------------------------*/
 
@@ -203,11 +220,14 @@ struct External {
   /*----------------------------------------------------------------------*/
 
   void push_external_clause_and_witness_on_extension_stack (
-      const vector<int> &clause, const vector<int> &witness);
+      const vector<int> &clause, const vector<int> &witness, uint64_t id);
+
+  void push_id_on_extension_stack (uint64_t id);
 
   // Restore a clause, which was pushed on the extension stack.
   void restore_clause (const vector<int>::const_iterator &begin,
-                       const vector<int>::const_iterator &end);
+                       const vector<int>::const_iterator &end,
+                       const uint64_t id);
 
   void restore_clauses ();
 
@@ -254,6 +274,12 @@ struct External {
 
   void reset_assumptions ();
 
+  // Similarly to 'failed', 'conclude' needs to know about failing
+  // assumptions and therefore needs to be reset when leaving the
+  // 'UNSATISFIED' state.
+  //
+  void reset_concluded ();
+
   // Similarly a valid external assignment obtained through 'extend' has to
   // be reset at each point it risks to become invalid.  This is done
   // in the external layer in 'external.cpp' functions..
@@ -281,16 +307,13 @@ struct External {
   //
   inline int ival (int elit) const {
     assert (elit != INT_MIN);
-    int eidx = abs (elit), res;
-    if (eidx > max_var)
-      res = -eidx;
-    else if ((size_t) eidx >= vals.size ())
-      res = -eidx;
-    else
-      res = vals[eidx] ? eidx : -eidx;
+    int eidx = abs (elit);
+    bool val = false;
+    if (eidx <= max_var && (size_t) eidx < vals.size ())
+      val = vals[eidx];
     if (elit < 0)
-      res = -res;
-    return res;
+      val = !val;
+    return val ? elit : -elit;
   }
 
   bool flip (int elit);
@@ -320,6 +343,11 @@ struct External {
 
   /*----------------------------------------------------------------------*/
 
+  int propagate_assumptions ();
+  void get_entrailed_literals (std::vector<int> &entrailed);
+  void conclude_unknown ();
+
+  /*----------------------------------------------------------------------*/
   int lookahead ();
   CaDiCaL::CubesWithStatus generate_cubes (int, int);
 
@@ -398,10 +426,12 @@ struct External {
     int eidx = abs (elit);
     if (eidx > max_var)
       return 0;
-    int res = solution[eidx];
+    signed char value = solution[eidx];
+    if (!value)
+      return 0;
     if (elit < 0)
-      res = -res;
-    return res;
+      value = -value;
+    return value > 0 ? elit : -elit;
   }
 };
 
