@@ -63,6 +63,7 @@ class IdrupTracer : public FileTracer {
   void idrup_conclude_and_delete (const vector<uint64_t> &conclusion);
   void idrup_report_status (int status);
   void idrup_conclude_sat (const vector<int> &model);
+  void idrup_conclude_unknown (const vector<int> &trail);
   void idrup_solve_query ();
 
 public:
@@ -81,6 +82,7 @@ public:
   void report_status (int, uint64_t) override;
   void conclude_sat (const vector<int> &) override;
   void conclude_unsat (ConclusionType, const vector<uint64_t> &) override;
+  void conclude_unknown (const vector<int> &) override;
 
   void solve_query () override;
   void add_assumption (int) override;
