@@ -117,7 +117,7 @@ class LratChecker : public StatTracer {
 
 public:
   LratChecker (Internal *);
-  ~LratChecker ();
+  virtual ~LratChecker ();
 
   void connect_internal (Internal *i) override;
   void begin_proof (uint64_t) override;
