@@ -1,25 +1,22 @@
 #ifndef _reap_h_INCLUDED
 #define _reap_h_INCLUDED
 
-#include <vector>
 #include <cstddef>
+#include <vector>
 
 class Reap {
 public:
-  Reap();
-  void init();
-  void release();
-  inline bool empty() {
-    return !num_elements;
-  }
+  Reap ();
+  void init ();
+  void release ();
+  inline bool empty () { return !num_elements; }
 
-  inline size_t size() {
-    return num_elements;
-  }
+  inline size_t size () { return num_elements; }
+
+  void push (unsigned);
+  void clear ();
+  unsigned pop ();
 
-  void push(unsigned);
-  void clear();
-  unsigned pop();
 private:
   size_t num_elements;
   unsigned last_deleted;
