@@ -79,6 +79,7 @@ run () {
     src=$tests/$1.cpp
     language=""
     COMPILE="$CXX $CXXFLAGS"
+    [ x"$1" = xparcompwrite ] && COMPILE="$COMPILE -pthread"
   else
     die "can not find '$tests.c' nor '$tests.cpp'"
   fi
@@ -100,6 +101,8 @@ run () {
 
 #--------------------------------------------------------------------------#
 
+run parcompwrite
+
 run newdelete
 run unit
 run morenmore
@@ -113,6 +116,7 @@ run cfreeze
 run traverse
 run cipasir
 run incproof
+run propagate_assumptions
 
 if [ "`grep DNTRACING $makefile`" = "" ]
 then
