@@ -49,7 +49,7 @@ struct WitnessChecker : WitnessIterator {
   }
 
 public:
-  bool witness (const vector<int> &c, const vector<int> &w) {
+  bool witness (const vector<int> &c, const vector<int> &w, uint64_t) {
     for (const auto &lit : w)
       cout << lit << ' ';
     cout << "0 ";
